/*
	NOMBRE DEL PAQUETE: PKG_CONTROL
	
  FUNCIONALIDAD: Proporciona las operaciones necesarias para almacenar las llamadas a procedimientos, sus parmetros
  de entrada y de salida en la tabla de LOG.

*/

spool 914_Implementacion_PKG_CONTROL.log;

------------------------------------------------
------------------- CABECERA -------------------
------------------------------------------------

CREATE OR REPLACE PACKAGE PKG_CONTROL IS
	NOMPAQUETE CHAR(15) := 'PKG_CONTROL';


/* 
   Actualiza el campo PARAMETROS_SALIDA del log correspondiente al id que recibe como parmetro de entrada.
   Se ejecuta de forma autnoma para garantizar su confirmacin en la base de datos independientemente de que 
   el procedimiento que lo llam haga un ROLLBACK.
   Incluye COMMIT slo para su ejecucin.
   @param e_log_id: id del log a modificar
   @param e_parametros: parmetros a incluir como parmetros de salida.
*/
PROCEDURE ACTUALIZA_LOG_SALIDA (
   e_id_log IN INTEGER
 , e_parametros IN VARCHAR2
); 


/* 
   Inserta una lnea en la tabla de LOGS asignando un nmero de registro e incluye los datos que recibe como
   parmetros de entrada, la fecha y hora actuales y el identificador de secuencia que le corresponda.
   Se ejecuta de forma autnoma para garantizar su confirmacin en la base de datos independientemente de que 
   el procedimiento que lo llam haga un ROLLBACK.
   Incluye COMMIT slo para su ejecucin.
   @param e_procedimiento: Nombre del procedimiento a incluir.
   @param e_parametros: Los parmetros de entrada de ese procedimiento.
   @return INTEGER: El LOG_ID creado en la insercin. 0 en caso de que no pueda insertar la lnea.
*/
FUNCTION INSERTA_LOG (
  e_procedimiento IN VARCHAR2
, e_parametros IN VARCHAR2
)
RETURN INTEGER;

END PKG_CONTROL;
/

----------------------------------------------
------------------- CUERPO -------------------
----------------------------------------------
CREATE OR REPLACE PACKAGE BODY PKG_CONTROL IS 

PROCEDURE ACTUALIZA_LOG_SALIDA 
(
   e_id_log IN INTEGER
 , e_parametros IN VARCHAR2
) AS 
PRAGMA AUTONOMOUS_TRANSACTION;
BEGIN
  
  -- Actualiza el campo PARAMETROS_SALIDA" en la tabla LOGS para el registro con el LOG_ID correspondiente
  UPDATE LOGS 
  SET "PARAMETROS_SALIDA" = substr(e_parametros,1,200)
  WHERE LOG_ID = e_id_log;
  
  COMMIT;
  
     
END ACTUALIZA_LOG_SALIDA;


FUNCTION INSERTA_LOG
(
  e_procedimiento IN VARCHAR2
, e_parametros IN VARCHAR2
)
RETURN INTEGER
AS
PRAGMA AUTONOMOUS_TRANSACTION;

-- Variable para almacenar el siguiente nmero de SEQ_LOG que dar el nmero para el LOG_ID
id_log NUMERIC;

BEGIN
 
  -- Toma el siguiente valor de la secuencia de nmeros
  id_log := seq_log.nextval;
  
  -- Inserta en la tabla LOGS los parmetros
  INSERT INTO LOGS (LOG_ID, FECHA, NOMBRE_PROCEDIMIENTO, PARAMETROS_ENTRADA)
  VALUES (id_log, SYSDATE, e_procedimiento, e_parametros);
   
  COMMIT;
  
  return id_log;
  
  
  -- En caso de que no pueda insertar los valores en la tabla de LOGS devolver 0
  -- y devolver SEQ_LOG a su valor anterior para la prxima insercin.
  EXCEPTION
    WHEN OTHERS THEN
      EXECUTE IMMEDIATE 'ALTER SEQUENCE  SEQ_LOG increment by -1';
      id_log := seq_log.nextval;
      EXECUTE IMMEDIATE 'ALTER SEQUENCE  SEQ_LOG increment by 1';
      RETURN 0;
   
END INSERTA_LOG;

END PKG_CONTROL;

/
spool off;