/*
	NOMBRE DEL PAQUETE: PKG_ADMINISTRACION
	
  FUNCIONALIDAD: Proporciona las operaciones de alta, baja y modificacin para los productos
  y las tiendas.
  Para los procesos de eliminacin se tiene que observar que se permite la eliminacin lo que 
  provoca la eliminacin en cascada de las ventas asociadas. Tras la eliminacin de las ventas
  se han de modificar las estadsticas de la tabla ESTADISTICAS_DIRECCION.

*/


spool 916_Implementacion_PKG_ADMINISTRACION.log;

------------------------------------------------
------------------- CABECERA -------------------
------------------------------------------------

CREATE OR REPLACE PACKAGE PKG_ADMINISTRACION IS
	NOMPAQUETE CHAR(20) := 'PKG_ADMINISTRACION';


/* 
   Inserta un producto en la tabla PRODUCTOS conforme a los parmetros que recibe.
   El procedimiento queda registrado en la tabla de LOGS.
   COMMIT INSIDE.
   @param e_cod_ean: Cdigo EAN13 del producto.
   @param e_descripcion: Descripcin abreviada del producto.
   @param e_fecha_incorporacion: Fecha de incorporacin del producto al catlogo.
   @param e_tipo: Tipo de producto conforme a las categoras de la compaa.
   @param e_genero: Alguno de la lista (Hombre, Mujer, Nio, Nia Unixex).
   @return rsp: "OK" si el producto se inserta correctamente.
                "ERROR. Ya existe el producto." Si el cdigo EAN13 ya existe en la tabla.
                "ERROR. No existe categora." Si la el tipo o gnero asignados al producto no existen.
                "ERROR + TIPO ERROR" si el producto no se puede insertar.
*/

PROCEDURE INSERTAR_PRODUCTO(
    e_cod_ean IN NUMBER
,   e_descripcion IN VARCHAR2
,   e_fecha_incorporacion IN DATE
,   e_tipo IN VARCHAR2
,   e_genero IN VARCHAR2
,   rsp OUT STRING
);


/*
   Modifica en la tabla PRODUCTOS la descripcin del producto cuyo cdigo EAN13 recibe como parmetro
   El procedimiento queda registrado en la tabla de LOGS.
   COMMIT INSIDE.
   @param e_cod_ean: Cdigo del producto del cual se quiere modificar la descripcin.
   @return rsp: "OK" si se modifica la descripcin correctamente.
                "ERROR. No existe producto." Si el producto a modificar no existe en la base de datos.
                "OK + TIPO ERROR" si no se puede modificar.
*/

PROCEDURE MODIFICAR_DESCRIPCION_PRODUCTO (
  e_cod_ean IN NUMBER
, e_descripcion IN VARCHAR2
, rsp OUT STRING
);


/*
   Elimina de la tabla PRODUCTOS el producto cuyo cdigo EAN13 recibe como parmetro.
   Se permite la eliminacin de un producto eliminando en cascada las ventas asociadas y reformulando las estadsticas de direccin.
   El procedimiento queda registrado en la tabla de LOGS.
   COMMIT INSIDE.
   @param e_cod_ean: Cdigo EAN13 del producto que se quiere eliminar.
   @return rsp: "OK" si el producto se elimina correctamente.
                "ERROR. No existe producto." Si el producto a eliminar no existe en la base de datos.
                "OK + TIPO ERROR" si no se puede eliminar.
*/
PROCEDURE ELIMINAR_PRODUCTO (
    e_cod_ean IN NUMBER
,   rsp OUT STRING
);


/* 
   Inserta una tienda en la tabla TIENDAS conforme a los parmetros que recibe.
   El procedimiento queda registrado en la tabla de LOGS.
   COMMIT INSIDE.
   @param e_nombre_ciudad: Ciudad donde se ubica la tienda
   @param e_nombre_region: Regin donde se ubica la ciudad
   @param e_fecha_correo_gerente: Direccin de correo electrnico del gerente de la tienda. Puede ser nulo
   @param e_franquicia: Rgimen de explotacin de la tienda. 0 Explotacin propia, 1 rgimen de franquicia
   @param e_virtual: 0 tienda fsica, 1 tienda virtual
   @param e_direccion_postal: Direccin postal de la tienda. Puede ser nulo
   @return rsp: "OK" si la tienda se inserta correctamente.
                "ERROR. Ya existe la tienda." Si el cdigo asignado ya existe en la tabla.
                "ERROR. No existe localizacin." Si la ciudad elegida no existe en la base de datos.                
                "OK + TIPO ERROR" si la tienda no se puede insertar.
*/

PROCEDURE INSERTAR_TIENDA (
  e_nombre_ciudad IN VARCHAR2
, e_nombre_region IN VARCHAR2
, e_correo_gerente IN VARCHAR2
, e_franquicia IN NUMBER
, e_virtual IN NUMBER
, e_direccion_postal IN VARCHAR
, rsp OUT STRING
);


/* 
   Modifica el campo DIRECCION_TIENDA de una tienda en la tabla TIENDAS conforme a los parmetros que recibe.
   El procedimiento queda registrado en la tabla de LOGS.
   COMMIT INSIDE.
   @param e_tiendad_id: Cdigo identificador de la tienda cuya direccin se quiere modificar.
   @param e_direccion_postal: Direccin postal de la tienda.
   @return rsp: "OK" si la tienda se modifica correctamente. 
                "ERROR. No existe tienda." Si la tienda a modificar no existe en la tabla.
                "OK + TIPO ERROR" si la tienda no se puede modificar.
*/
PROCEDURE MODIFICAR_DIRECCION_TIENDA (
  e_tienda_id IN NUMBER
, e_direccion_postal IN VARCHAR2
, rsp OUT STRING
);


/* 
   Modifica el campo CORREO_GERENTE de una tienda en la tabla TIENDAS conforme a los parmetros que recibe.
   El procedimiento queda registrado en la tabla de LOGS.
   COMMIT INSIDE.
   @param e_tiendad_id: Cdigo identificador de la tienda cuyo correo se quiere modificar.
   @param e_correo: Correo electrnico del gerente de la tienda.   
   @return rsp: "OK" si la tienda se modifica correctamente.
                "ERROR. No existe tienda." Si la tienda a modificar no existe en la tabla.
                "OK + TIPO ERROR" si la tienda no se puede modificar.
*/
PROCEDURE MODIFICAR_CORREO_TIENDA (
  e_tienda_id IN NUMBER
, e_correo IN VARCHAR2
, rsp OUT STRING
);


/*
   Elimina de la tabla TIENDAS la tienda cuyo cdigo de identificacin recibe como parmetro
   Se permite la eliminacin de una tienda eliminando en cascada las ventas asociadas y reformulando las estadsticas de direccin.
   El procedimiento queda registrado en la tabla de LOG
   COMMIT INSIDE.
   @param e_cod_ean: Cdigo identificador de la tienda que se quiere eliminar.
   @return rsp: "OK" si la tienda se elimina correctamente.
                "ERROR. No existe tienda." Si la tienda a modificar no existe en la tabla.
                "OK + TIPO ERROR" si no se puede eliminar.
*/
PROCEDURE ELIMINAR_TIENDA(
  e_tienda_id IN NUMBER
, rsp OUT STRING
);


end PKG_ADMINISTRACION;
/

----------------------------------------------
------------------- CUERPO -------------------
----------------------------------------------
CREATE OR REPLACE PACKAGE BODY PKG_ADMINISTRACION IS 

PROCEDURE INSERTAR_PRODUCTO(
    e_cod_ean IN NUMBER
,   e_descripcion IN VARCHAR2
,   e_fecha_incorporacion IN DATE
,   e_tipo IN VARCHAR2
,   e_genero IN VARCHAR2
,   rsp OUT STRING
)

AS

-- Variable para almacenar el valor del LOG_ID que le corresponda a la llamada al procedimiento.
id_log INTEGER;
-- Excepcin en el caso de que falle el registro de LOG.
problema_log EXCEPTION;
-- Excepcin para el caso de que no haya tipo o gnero.
problema_categoria EXCEPTION;
PRAGMA EXCEPTION_INIT(problema_categoria, -2291);

BEGIN

   -- Registra en la tabla de LOGS la llamada al procedimiento.   
   id_log := PKG_CONTROL.INSERTA_LOG('INSERTAR_PRODUCTO', 'EAN13: '||e_cod_ean||', Descripcion: '||e_descripcion||', Fecha incorporacion: '
   ||e_fecha_incorporacion||', Tipo: '||e_tipo||', Genero: '||e_genero); 

   -- Si hay algn problema al insertar el LOG lanza una excepcin definida por el usuario.
   -- Si no hay problema al insertar el LOG prosigue con la insercin.
   IF id_log = 0 THEN
      RAISE problema_log;
   
   -- Inserta el producto en la tabla PRODUCTOS.
   ELSE  
      INSERT INTO PRODUCTOS (COD_EAN, DESCRIPCION, FECHA_INCORPORACION, TIPO, GENERO)
      VALUES(e_cod_ean, e_descripcion, e_fecha_incorporacion, e_tipo, e_genero);
   END IF;
  
   -- Si la insercin se ha efectuado correctamente da el valor OK a rsp y actualiza el registro del log.    
   rsp :='OK';
   PKG_CONTROL.ACTUALIZA_LOG_SALIDA (id_log,rsp);
   
   -- Confirma los cambios.
   COMMIT;
      
   EXCEPTION
       -- Si no se ha podido abrir registro para el log informa al usuario. 
       WHEN problema_log THEN
         rsp := 'ERROR. No se puede guardar el log de la transaccin.';
       -- Si ya existe el cdigo EAN asignado.
       WHEN DUP_VAL_ON_INDEX THEN
         rsp := 'ERROR. Ya existe el producto.';
         PKG_CONTROL.ACTUALIZA_LOG_SALIDA (id_log,rsp);       
       -- Si la categora asignada no existe. 
       WHEN problema_categoria THEN
         rsp := 'ERROR. No existe categora.';
         PKG_CONTROL.ACTUALIZA_LOG_SALIDA (id_log,rsp);           
      --Para cualquier otro error informa en la tabla de LOGS del error.
       WHEN OTHERS THEN
         rsp:='ERROR ' || SUBSTR(SQLERRM,1,200);
         PKG_CONTROL.ACTUALIZA_LOG_SALIDA (id_log,rsp);
          
          
END INSERTAR_PRODUCTO;


PROCEDURE MODIFICAR_DESCRIPCION_PRODUCTO (
  e_cod_ean IN NUMBER
, e_descripcion IN VARCHAR2
, rsp OUT STRING
)AS

-- Variable para almacenar el valor del LOG_ID que le corresponda a la llamada al procedimiento.
id_log INTEGER;
-- Excepcin para el caso en que falle el registro de LOG.
problema_log EXCEPTION;
-- Excepcion para cuando el producto a modificar no existe en la base de datos.
no_existe_producto EXCEPTION;

BEGIN

   -- Registra en la tabla de LOGS la llamada al procedimiento.   
   id_log := PKG_CONTROL.INSERTA_LOG('MODIFICAR_DESCRIPCION_PRODUCTO', 'Producto: '||e_cod_ean||', Descripcion: '||e_descripcion); 

   -- Si hay algn problema al insertar el LOG lanza una excepcin definida por el usuario.
   -- Si no hay problema al insertar el LOG prosigue con la modificacin.
   IF id_log = 0 THEN
      RAISE problema_log;
   
   ELSE  
      -- Actualiza el registro.
      UPDATE PRODUCTOS SET DESCRIPCION = e_descripcion
      WHERE COD_EAN = e_cod_ean;
   
      -- Si no ha actualizado nada lanza la excepcin.
      IF sql%ROWCOUNT = 0 THEN
         RAISE no_existe_producto;
      END IF;
   
   END IF;
   
   -- Si la actualizacin se ha efectuado correctamente da el valor OK a rsp y actualiza el registro de log.    
   rsp :='OK';
   PKG_CONTROL.ACTUALIZA_LOG_SALIDA (id_log,rsp);
   
   COMMIT;
      
   EXCEPTION
       -- Si no se ha podido abrir registro para el log informa al usuario 
       WHEN problema_log THEN
          rsp:= 'ERROR. No se puede guardar el log de la transaccin.';
       -- Si el producto a actualizar no existe en la tabla.
       WHEN no_existe_producto THEN
          rsp := 'ERROR. No existe producto.';
          PKG_CONTROL.ACTUALIZA_LOG_SALIDA (id_log,rsp);       
       --Para cualquier otro error informa en la tabla de LOGS del error
       WHEN OTHERS THEN
          rsp := 'ERROR ' || SUBSTR(SQLERRM,1,200);
          PKG_CONTROL.ACTUALIZA_LOG_SALIDA (id_log,rsp);

END MODIFICAR_DESCRIPCION_PRODUCTO;


PROCEDURE ELIMINAR_PRODUCTO (
    e_cod_ean IN NUMBER
,   rsp OUT STRING
)
AS

-- Variable para almacenar el valor del LOG_ID que le corresponda a la llamada al procedimiento.
id_log INTEGER;
rsp_temp STRING (200);
-- Variable para lanzar Excepcin en el caso de que falle el registro de LOG.
problema_log EXCEPTION;
no_existe_producto EXCEPTION;

BEGIN

   -- Registra en la tabla de LOGS la llamada al procedimiento.   
   id_log := PKG_CONTROL.INSERTA_LOG('ELIMINAR_PRODUCTO', 'Producto: '||e_cod_ean); 

   -- Si hay algn problema al insertar el LOG lanza una excepcin definida por el usuario.
   -- Si no hay problema al insertar el LOG prosigue con la eliminacin.
   IF id_log = 0 THEN
      RAISE problema_log;
   
   ELSE  
      -- Elimina el producto de la tabla.
      DELETE FROM PRODUCTOS WHERE COD_EAN = e_cod_ean;
   
      -- Si no ha podido eliminar el producto.
      IF sql%ROWCOUNT = 0  THEN
         RAISE no_existe_producto;
      END IF;
  
  END IF;
   -- Si la eliminacin se ha efectuado correctamente da el valor OK a rsp y actualiza el registro de log.    
   rsp :='OK';
   PKG_CONTROL.ACTUALIZA_LOG_SALIDA (id_log,rsp);
       
   COMMIT;
      
   EXCEPTION
       -- Si no se ha podido abrir registro para el log informa al usuario 
       WHEN problema_log THEN
          rsp:= 'ERROR. No se puede guardar el log de la transaccin.';
       -- Si el producto no existe en la tabla.
       WHEN no_existe_producto THEN
          rsp:='ERROR. El producto no existe.';
          PKG_CONTROL.ACTUALIZA_LOG_SALIDA (id_log,rsp);
       --Para cualquier otro error informa en la tabla de LOGS del error.
       WHEN OTHERS THEN
          rsp:='ERROR ' || SUBSTR(SQLERRM,1,200);
          PKG_CONTROL.ACTUALIZA_LOG_SALIDA (id_log,rsp);

END ELIMINAR_PRODUCTO;


PROCEDURE INSERTAR_TIENDA (
  e_nombre_ciudad IN VARCHAR2
, e_nombre_region IN VARCHAR2
, e_correo_gerente IN VARCHAR2
, e_franquicia IN NUMBER
, e_virtual IN NUMBER
, e_direccion_postal IN VARCHAR
, rsp OUT STRING
) AS

-- Variable para almacenar el valor del LOG_ID que le corresponda a la llamada al procedimiento.
id_log INTEGER;
-- Variable para lanzar Excepcin en el caso de que falle el registro de LOG.
problema_log EXCEPTION;
-- Excepcin para el caso de que no haya ciudad.
problema_categoria EXCEPTION;
PRAGMA EXCEPTION_INIT(problema_categoria, -2291);


BEGIN

   -- Registra en la tabla de LOGS la llamada al procedimiento.   
   id_log := PKG_CONTROL.INSERTA_LOG('INSERTAR_TIENDA', 'Ciudad: '||e_nombre_ciudad||', Correo gerente: '
   ||e_correo_gerente||', Franquicia: '||e_franquicia||', Virtual: '||e_virtual||', Direccion: '||e_direccion_postal); 

   -- Si hay algn problema al insertar el LOG lanza una excepcin definida por el usuario
   -- Si no hay problema al insertar el LOG prosigue con la insercin
   IF id_log = 0 THEN
      RAISE problema_log;
   
   ELSE   
      -- Inserta la tienda en la tabla TIENDAS.
      INSERT INTO TIENDAS (TIENDA_ID, NOMBRE_CIUDAD, NOMBRE_REGION, CORREO_GERENTE,
      FRANQUICIA, VIRTUAL, DIRECCION_POSTAL)
      VALUES (SEQ_TIENDAS.NEXTVAL, e_nombre_ciudad, e_nombre_region, e_correo_gerente, 
      e_franquicia, e_virtual, e_direccion_postal);

   END IF;
   
   -- Si la insercin se ha efectuado correctamente da el valor OK  rsp y actualiza el registro de log.    
   rsp :='OK';
   PKG_CONTROL.ACTUALIZA_LOG_SALIDA (id_log,rsp);
   
   COMMIT;
      
   EXCEPTION
       -- Si no se ha podido abrir registro para el log informa al usuario.
       WHEN problema_log THEN
          rsp:= 'ERROR. No se puede guardar el log de la transaccin.';
       -- Si el cdigo asignado a la tienda ya existe.
       WHEN DUP_VAL_ON_INDEX THEN
         rsp:= 'ERROR. Ya existe la tienda.';
         PKG_CONTROL.ACTUALIZA_LOG_SALIDA (id_log,rsp);      
       -- Si no existe la ciudad de la tienda.
       WHEN problema_categoria THEN
         rsp := 'ERROR. No existe localizacin.';
         PKG_CONTROL.ACTUALIZA_LOG_SALIDA (id_log,rsp);        
       --Para cualquier otro error informa en la tabla de LOGS del error.
       WHEN OTHERS THEN
          rsp:='ERROR ' || SUBSTR(SQLERRM,1,200);
          PKG_CONTROL.ACTUALIZA_LOG_SALIDA (id_log,rsp);


END INSERTAR_TIENDA;


PROCEDURE MODIFICAR_DIRECCION_TIENDA (
  e_tienda_id IN NUMBER
, e_direccion_postal IN VARCHAR2
, rsp OUT STRING
) AS

-- Variable para almacenar el valor del LOG_ID que le corresponda a la llamada al procedimiento.
id_log INTEGER;
-- Variable para lanzar Excepcin en el caso de que falle el registro de LOG.
problema_log EXCEPTION;
-- Excepcion para cuando la tienda a modificar no existe en la base de datos.
no_existe_tienda EXCEPTION;

BEGIN

   -- Registra en la tabla de LOGS la llamada al procedimiento   
   id_log := PKG_CONTROL.INSERTA_LOG('MODIFICAR_DIRECCION_TIENDA', 'Tienda: '||e_tienda_id||', Direccion: '||e_direccion_postal); 

   -- Si hay algn problema al insertar el LOG lanza una excepcin definida por el usuario.
   -- Si no hay problema al insertar el LOG prosigue con la modificacin.
   IF id_log = 0 THEN
      RAISE problema_log;
   
   ELSE  
      -- Actualiza el registro.
      UPDATE TIENDAS SET DIRECCION_POSTAL = e_direccion_postal
      WHERE TIENDA_ID = e_tienda_id;

      -- Si no se puede actualizar el registro.     
      IF sql%ROWCOUNT = 0 THEN
        RAISE no_existe_tienda;
      END IF;
   
   END IF;

   -- Si la actualizacin se ha efectuado correctamente da el valor OK a rsp y actualiza el registro de log.    
   rsp :='OK';
   PKG_CONTROL.ACTUALIZA_LOG_SALIDA (id_log,rsp);
   
   COMMIT;
      
   EXCEPTION
       -- Si no se ha podido abrir registro para el log informa al usuario. 
       WHEN problema_log THEN
          rsp:= 'ERROR. No se puede guardar el log de la transaccin.';
       -- Si la tienda no existe en la TABLA.
       WHEN no_existe_tienda THEN
          rsp := 'ERROR. No existe tienda.';
          PKG_CONTROL.ACTUALIZA_LOG_SALIDA (id_log,rsp); 
       --Para cualquier otro error informa en la tabla de LOGS del error.
       WHEN OTHERS THEN
          rsp:='ERROR ' || SUBSTR(SQLERRM,1,200);
          PKG_CONTROL.ACTUALIZA_LOG_SALIDA (id_log,rsp);

END MODIFICAR_DIRECCION_TIENDA;


PROCEDURE MODIFICAR_CORREO_TIENDA (
  e_tienda_id IN NUMBER
, e_correo IN VARCHAR2
, rsp OUT STRING
) AS

-- Variable para almacenar el valor del LOG_ID que le corresponda a la llamada al procedimiento.
id_log INTEGER;
-- Variable para lanzar Excepcin en el caso de que falle el registro de LOG.
problema_log EXCEPTION;
-- Excepcion para cuando la tienda a modificar no existe en la base de datos.
no_existe_tienda EXCEPTION;

BEGIN

   -- Registra en la tabla de LOGS la llamada al procedimiento.   
   id_log := PKG_CONTROL.INSERTA_LOG('MODIFICAR_CORREO_TIENDA', 'Tienda: '||e_tienda_id||', Correo: '||e_correo); 

   -- Si hay algn problema al insertar el LOG lanza una excepcin definida por el usuario.
   -- Si no hay problema al insertar el LOG prosigue con la modificacin.
   IF id_log = 0 THEN
      RAISE problema_log;
   
   ELSE  
      -- Actualiza el registro.      
      UPDATE TIENDAS SET CORREO_GERENTE = e_correo
      WHERE TIENDA_ID = e_tienda_id;

      -- Si no se puede actualizar el registro.
      IF sql%ROWCOUNT = 0 THEN
        RAISE no_existe_tienda;
      END IF;
      
   END IF;

   -- Si la actualizacin se ha efectuado correctamente da el valor OK a rsp y actualiza el registro de log.    
   rsp :='OK';
   PKG_CONTROL.ACTUALIZA_LOG_SALIDA (id_log,rsp);
   
   COMMIT;
      
   EXCEPTION
       -- Si no se ha podido abrir registro para el log informa al usuario. 
       WHEN problema_log THEN
          rsp:= 'ERROR. No se puede guardar el log de la transaccin.';
       -- Si la tienda no existe en la TABLA.
       WHEN no_existe_tienda THEN
          rsp := 'ERROR. No existe tienda.';
          PKG_CONTROL.ACTUALIZA_LOG_SALIDA (id_log,rsp); 
       --Para cualquier otro error informa en la tabla de LOGS del error.       
       WHEN OTHERS THEN
          rsp:='ERROR ' || SUBSTR(SQLERRM,1,200);
          PKG_CONTROL.ACTUALIZA_LOG_SALIDA (id_log,rsp);

END MODIFICAR_CORREO_TIENDA;


PROCEDURE ELIMINAR_TIENDA(
  e_tienda_id IN NUMBER
, rsp OUT STRING
)AS

-- Variable para almacenar el valor del LOG_ID que le corresponda a la llamada al procedimiento.
id_log INTEGER;
rsp_temp STRING (200);
-- Variable para lanzar Excepcin en el caso de que falle el registro de LOG.
problema_log EXCEPTION;
no_existe_tienda EXCEPTION;

BEGIN

   -- Registra en la tabla de LOGS la llamada al procedimiento.   
   id_log := PKG_CONTROL.INSERTA_LOG('ELIMINAR_TIENDA', 'Tienda: '||e_tienda_id); 

   -- Si hay algn problema al insertar el LOG lanza una excepcin definida por el usuario.
   -- Si no hay problema al insertar el LOG prosigue con la eliminacin.
   IF id_log = 0 THEN
      RAISE problema_log;
   
   ELSE  
      -- Elimina el registro de la tabla TIENDAS.
      DELETE FROM TIENDAS WHERE TIENDA_ID = e_tienda_id;
  
      -- Si no ha podido eliminar el registro.
      IF sql%ROWCOUNT = 0  THEN
         RAISE no_existe_tienda;
      END IF;
   
   END IF;
   -- Si la eliminacin se ha efectuado correctamente da el valor OK  rsp y actualiza el registro de log    
   rsp :='OK';
   PKG_CONTROL.ACTUALIZA_LOG_SALIDA (id_log,rsp);
   
   COMMIT;

   EXCEPTION
       -- Si no se ha podido abrir registro para el log informa al usuario. 
       WHEN problema_log THEN
          rsp:= 'ERROR. No se puede guardar el log de la transaccin.';
       -- Si la tienda no existe en la TABLA.
       WHEN no_existe_tienda THEN
          rsp := 'ERROR. No existe tienda.';
          PKG_CONTROL.ACTUALIZA_LOG_SALIDA (id_log,rsp); 
       --Para cualquier otro error informa en la tabla de LOGS del error.
       WHEN OTHERS THEN
          rsp:='ERROR ' || SUBSTR(SQLERRM,1,200);
          PKG_CONTROL.ACTUALIZA_LOG_SALIDA (id_log,rsp);

END ELIMINAR_TIENDA;

END PKG_ADMINISTRACION;

/
spool off;











