/*
	NOMBRE DEL PAQUETE: PKG_VENTA
	
  FUNCIONALIDAD: Proporciona las operaciones necesarias para almacenar las ventas de las tiendas de la cadena.

*/

spool 917_Implementacion_PKG_VENTA.log;

------------------------------------------------
------------------- CABECERA -------------------
------------------------------------------------

CREATE OR REPLACE PACKAGE PKG_VENTA IS
	NOMPAQUETE CHAR(20) := 'PKG_VENTA';

/* 
   Obtiene la cantidad total, el beneficio neto acumulado y el precio venta bruto acumulado de las ventas del producto
   cuyo cdigo EAN13 se le pasa como parmetro para una fecha y hora determinados.
   @param e_cod_ean: Cdigo del artculo buscado.
   @param e_fecha: La fecha para la cual se quiere conocer los datos.
   @param e_hora: La hora para la cual se quiere conocer los datos.
   @return T_VENTAS (Table of numbers) incluyendo
                         (1) La cantidad de unidades
                         (2) El beneficio neto acumulado
                         (3) El precio de venta bruto acumulado
            Si no encuentra valores devuelve (0,0,0)
*/
FUNCTION OBTENER_DATOS_VENTA_PRODUCTO (
   e_cod_ean IN NUMBER 
,  e_fecha IN STRING
,  e_hora IN NUMBER
)
RETURN T_VENTA;

/* 
   Inserta una venta en la tabla VENTAS conforme a los parmetros que recibe.
   El procedimiento queda registrado en la tabla de LOGS.
   COMMIT INSIDE.
   ROLLBACK si no termina correctamente.
   @param e_tienda_id: Cdigo de la tienda que efecta la venta.
   @param e_cod_ean: Cdigo del artculo vendido.
   @param e_unidades_vendidas: Unidades del producto vendidas en esa transaccin.
   @param e_precio_bruto: Precio bruto unitario del producto vendido.
   @param e_beneficio_neto: Beneficio neto unitario del producto vendido.
   @return rsp: "OK" si la venta se inserta correctamente. OK + TIPO ERROR si el producto no se puede insertar.
                "ERROR No existe la tienda." Si no existe la tienda.
                "ERROR No existe el producto." Si no existe el producto. 
                "ERROR No se puede guardar registro de transaccin." Si hay algn problema con el registro log.
                "ERROR + TIPO ERROR" En otro caso.
*/
PROCEDURE INSERTAR_VENTA (
    e_tienda_id IN NUMBER 
,   e_cod_ean IN NUMBER  
,   e_ud_vendidas IN NUMBER  
,   e_precio_bruto IN NUMBER 
,   e_beneficio_neto IN NUMBER 
,   rsp OUT STRING
);

/*
    Utiliza la funcin INSERTAR_VENTA forzando la fecha de entrada de la venta para que no sea automtica y se puedan hacer
    las pruebas de los informes del PKG_EST_DIR.
    Este procedimiento no se registra en la tabla de LOGS.
    COMMIT INSIDE.
    ROLLBACK si no termina correctamente.
    @param e_tienda_id: Cdigo de la tienda que efecta la venta
    @param e_cod_ean: Cdigo del artculo vendido
    @param e_fecha: La fecha de la venta
    @param e_hora: La hora de la venta
    @param e_unidades_vendidas: Unidades del producto vendidas en esa transaccin
    @param e_precio_bruto: Precio bruto unitario del producto vendido
    @param e_beneficio_neto: Beneficio neto unitario del producto vendido
    @return rsp: Ok si la venta se inserta correctamente. OK + TIPO ERROR si el producto no se puede insertar.
    @return rsp: "OK" si la venta se inserta correctamente. OK + TIPO ERROR si el producto no se puede insertar.
                 "ERROR No existe la tienda." Si no existe la tienda.
                 "ERROR No existe el producto." Si no existe el producto. 
                 "ERROR No se puede guardar registro de transaccin." Si hay algn problema con el registro log.
                 "ERROR + TIPO ERROR" En otro caso.
*/
PROCEDURE INSERTAR_VENTA_PRUEBAS (
    e_tienda_id IN NUMBER 
,   e_cod_ean IN NUMBER  
,   e_fecha IN DATE
,   e_hora IN NUMBER
,   e_ud_vendidas IN NUMBER 
,   e_precio_bruto IN NUMBER 
,   e_beneficio_neto IN NUMBER 
,   rsp OUT STRING
);

END PKG_VENTA;
/

----------------------------------------------
------------------- CUERPO -------------------
----------------------------------------------
CREATE OR REPLACE PACKAGE BODY PKG_VENTA IS 

FUNCTION OBTENER_DATOS_VENTA_PRODUCTO (
   e_cod_ean IN NUMBER 
,  e_fecha IN STRING
,  e_hora IN NUMBER
)
 
RETURN T_VENTA

AS 

     -- Variable para almacenar el resultado de la consulta.
     -- Inicializa el tipo con el constructor por defecto.
     datos_venta_producto T_VENTA := T_VENTA(); 

BEGIN

     -- Se da capacidad para tres valores.
     datos_venta_producto.EXTEND(3); 
     
     -- Selecciona la ltima de las ventas de ese producto en esa fecha y hora.
     -- Segn las especificaciones en cada venta se guarda en CTD_TOTAL la cantidad acumulada de unidades vendidas de ese producto para esa fecha y hora.
     -- Segn las especificaciones en cada venta se guarda en PB_TOTAL el precio bruto acumulado de las ventas de ese producto para esa fecha y hora.
     -- Segn las especificaciones en cada venta se guarda en BN_TOTAL el beneficio neto acumulado de las ventas de ese producto para esa fecha y hora.
     SELECT CTD_TOTAL, PB_TOTAL, BN_TOTAL 
     INTO datos_venta_producto(1), datos_venta_producto(2), datos_venta_producto(3)  
     FROM (select * from "VENTAS"
           WHERE COD_EAN = e_cod_ean AND FECHA = TO_DATE(e_fecha) and HORA = e_hora
           ORDER BY "VENTA_ID" DESC)
     WHERE ROWNUM = 1;
     
     RETURN datos_venta_producto;

     EXCEPTION
      WHEN NO_DATA_FOUND THEN
        datos_venta_producto (1) :=0;
        datos_venta_producto (2) :=0;
        datos_venta_producto (3) :=0;
        RETURN datos_venta_producto;
      WHEN OTHERS THEN
        RAISE;
     
END OBTENER_DATOS_VENTA_PRODUCTO;


PROCEDURE INSERTAR_VENTA (
    e_tienda_id IN NUMBER 
,   e_cod_ean IN NUMBER  
,   e_ud_vendidas IN NUMBER  
,   e_precio_bruto IN NUMBER 
,   e_beneficio_neto IN NUMBER 
,   rsp OUT STRING
) AS

  -- Variable de tipo "table of number" para registrar la consulta sobre determinados valores de ventas pasadas del producto.
  datos_venta_producto T_VENTA;
  -- Variable para almacenar la fecha actual del sistema.
  fecha_actual DATE := TO_DATE (SYSDATE); 
  -- Variable para almacenar la hora actual del sistema.
  hora_actual NUMBER := TO_CHAR(SYSDATE,'HH24'); 
  -- Variable para almacenar el valor del LOG_ID que le corresponda a la llamada al procedimiento.
  id_log INTEGER;
  -- Variable temporal para almacenar el resultado de determinadas bsquedas en tablas TIENDAS y PRODUCTOS.
  existe_consulta INTEGER;
  -- Variable temporal para almacenar el resultado de bsquedas de fecha en la tabla FECHAS.
  existe_dia DATE;
  rsp_temp STRING (200);
  
   -- Excepcin en el caso de que no existe la tienda.
  no_existe_tienda EXCEPTION;
  -- Excepcin en el caso de que no existe el producto.
  no_existe_producto EXCEPTION;
  -- Variable para lanzar excepcin en el caso de que falle el registro de LOG.
  problema_log EXCEPTION;
  

BEGIN

      -- Registra en la tabla de LOGS la llamada al procedimiento.
      id_log := PKG_CONTROL.INSERTA_LOG('INSERTAR_VENTA','Tienda: '||e_tienda_id||', Producto: '||e_cod_ean||', Unidades vendidas: '||e_ud_vendidas
      ||', Precio Bruto: '||e_precio_bruto||', Beneficio Neto: '||e_beneficio_neto); 
          
      -- Si hay algn problema al insertar el LOG lanza una excepcin definida por el usuario.
      -- Si no hay problema al insertar el LOG prosigue con la insercin.
      if id_log = 0 then
        raise problema_log;
      else
        
        -- Busca en la tabla TIENDAS la tienda con el identificador pasado como parmetro. Si no la encuentra lanza una excepcin.        
        BEGIN
          SELECT TIENDA_ID INTO existe_consulta FROM TIENDAS WHERE TIENDA_ID = e_tienda_id;
          EXCEPTION 
            WHEN NO_DATA_FOUND THEN
             raise no_existe_tienda;
        END;

        -- Busca en la tabla PRODUCTOS el producto con el cdigo EAN13 pasado como parmetro. Si no lo encuentra lanza una excepcin.
        BEGIN
          SELECT COD_EAN INTO existe_consulta FROM PRODUCTOS WHERE COD_EAN = e_cod_ean;
          EXCEPTION 
           WHEN NO_DATA_FOUND THEN
             raise no_existe_producto;
        END;

        -- Busca en la tabla FECHAS la fecha y hora pasadas como parmetro. Si no los encuentra inserta en dicha tabla la fecha y hora actuales.
        BEGIN
          SELECT FECHA, HORA INTO existe_dia, existe_consulta FROM FECHAS
          WHERE FECHA = fecha_actual AND HORA = hora_actual;
          EXCEPTION 
            WHEN NO_DATA_FOUND THEN
              INSERT INTO FECHAS ("FECHA", "HORA") VALUES (fecha_actual, hora_actual);
        END;
     
        -- En caso de que todos los datos existan se consultan los datos histricos sobre cantidad actual vendida,
        -- el precio bruto acumulado y el beneficio neto acumulado para ese producto, fecha y hora. Se consideran slo 
        -- ventas realizadas por lo que la venta que se est introduciendo no est incluida en los clculos.
        datos_venta_producto := OBTENER_DATOS_VENTA_PRODUCTO (e_cod_ean, fecha_actual, hora_actual);
     
         -- Inserta la venta en la tabla de ventas.
        INSERT INTO VENTAS ("VENTA_ID", "TIENDA_ID", "COD_EAN", "FECHA", "HORA", "UD_VENDIDAS", "CTD_TOTAL", "PB_TOTAL",
        "BN_TOTAL","PB_UNITARIO","BN_UNITARIO")
        VALUES (seq_ventas.nextval, e_tienda_id, e_cod_ean, fecha_actual, hora_actual, e_ud_vendidas, (datos_venta_producto(1)+ e_ud_vendidas),
        (datos_venta_producto (2)+ e_precio_bruto * e_ud_vendidas), (datos_venta_producto(3)+ e_beneficio_neto * e_ud_vendidas),
        e_precio_bruto, e_beneficio_neto);
     
        -- Actualiza las estadsticas de direccin.
        PKG_EST_DIR.ACTLZR_EST_TOTAL(e_tienda_id, e_cod_ean, fecha_actual, hora_actual, e_ud_vendidas,e_precio_bruto,e_beneficio_neto, rsp_temp);
        PKG_EST_DIR.ACTLZR_EST_ANUAL(e_tienda_id, e_cod_ean, fecha_actual, hora_actual, e_ud_vendidas,e_precio_bruto,e_beneficio_neto, rsp_temp);
     

        -- Si la insercin se ha efectuado correctamente da el valor OK a rsp y actualiza el registro de log.    
        rsp :='OK';
        PKG_CONTROL.ACTUALIZA_LOG_SALIDA (id_log,rsp);

        COMMIT;
    
      END IF;

      EXCEPTION
       -- Si no se ha podido abrir registro para el log informa al usuario      
      WHEN problema_log THEN
        rsp:= 'ERROR. No se puede guardar registro de transaccin.';
      -- Si no existe la tienda.
      WHEN no_existe_tienda THEN
        rsp:='ERROR. No existe la tienda.';
        PKG_CONTROL.ACTUALIZA_LOG_SALIDA (id_log,rsp);
      -- sin no existe el producto lanza en error y avisa de este hecho.
      WHEN no_existe_producto THEN
        rsp:='ERROR. No existe el producto.';
        PKG_CONTROL.ACTUALIZA_LOG_SALIDA (id_log,rsp);
      WHEN OTHERS THEN
        rsp:='ERROR ' || SUBSTR(SQLERRM,1,200);
        ROLLBACK; 
        PKG_CONTROL.ACTUALIZA_LOG_SALIDA (id_log,rsp);

END INSERTAR_VENTA;


PROCEDURE INSERTAR_VENTA_PRUEBAS (
    e_tienda_id IN NUMBER 
,   e_cod_ean IN NUMBER  
,   e_fecha IN DATE
,   e_hora IN NUMBER
,   e_ud_vendidas IN NUMBER  
,   e_precio_bruto IN NUMBER 
,   e_beneficio_neto IN NUMBER 
,   rsp OUT STRING
)
AS

  -- Variable de tipo "table of number" para registrar la consulta sobre determinados valores de ventas pasadas del producto.
  datos_venta_producto T_VENTA;
  -- Variable para almacenar la fecha actual del sistema.
  fecha_actual DATE := e_fecha; 
  -- Variable para almacenar la hora actual del sistema.
  hora_actual NUMBER := e_hora; 
  -- Variable para almacenar el valor del LOG_ID que le corresponda a la llamada al procedimiento.
  id_log INTEGER;
  -- Variable temporal para almacenar el resultado de determinadas bsquedas en tablas TIENDAS y PRODUCTOS.
  existe_consulta INTEGER;
  -- Variable temporal para almacenar el resultado de bsquedas de fecha en la tabla FECHAS.  
  existe_dia DATE;
  rsp_temp STRING (200);
  
   -- Excepcin en el caso de que no existe la tienda
  no_existe_tienda EXCEPTION;
  -- Excepcin en el caso de que no existe el producto
  no_existe_producto EXCEPTION;
  -- Variable para lanzar excepcin en el caso de que falle el registro de LOG
  problema_log EXCEPTION;
  

BEGIN

      -- Registra en la tabla de LOGS la llamada al procedimiento.
      id_log := PKG_CONTROL.INSERTA_LOG('INSERTAR_VENTA_PRUEBAS','Tienda: '||e_tienda_id||', Producto: '||e_cod_ean||', Unidades vendidas: '||e_ud_vendidas
      ||', Precio Bruto: '||e_precio_bruto||', Beneficio Neto: '||e_beneficio_neto); 
          
      -- Si hay algn problema al insertar el LOG lanza una excepcin definida por el usuario.
      -- Si no hay problema al insertar el LOG prosigue con la insercin.
      if id_log = 0 then
        raise problema_log;
      else
        
        -- Busca en la tabla TIEDAS la tienda con el identificador pasado como parmetro. Si no la encuentra lanza una excepcin.        
        BEGIN
          SELECT TIENDA_ID INTO existe_consulta FROM TIENDAS WHERE TIENDA_ID = e_tienda_id;
          EXCEPTION 
            WHEN NO_DATA_FOUND THEN
              raise no_existe_tienda;
        END;

        -- Busca en la tabla PRODUCTOS el producto con el cdigo EAN13 pasado como parmetro. Si no lo encuentra lanza una excepcin.
        BEGIN
          SELECT COD_EAN INTO existe_consulta FROM PRODUCTOS WHERE COD_EAN = e_cod_ean;
          EXCEPTION 
            WHEN NO_DATA_FOUND THEN
              raise no_existe_producto;
        END;

        -- Busca en la tabla FECHAS la fecha y hora pasadas como parmetro. Si no los encuentra inserta en dicha tabla la fecha y hora actuales.
        BEGIN
          SELECT FECHA, HORA INTO existe_dia, existe_consulta FROM FECHAS
          WHERE FECHA = fecha_actual AND HORA = hora_actual;
          EXCEPTION 
            WHEN NO_DATA_FOUND THEN
              INSERT INTO FECHAS ("FECHA", "HORA") VALUES (fecha_actual, hora_actual);
        END;
     
        -- En caso de que todos los datos existan se consultan los datos histricos sobre cantidad actual vendida,
        -- el precio bruto acumulado y el beneficio neto acumulado para ese producto, fecha y hora. Se consideran slo 
        -- ventas realizadas por lo que la venta que se est introduciendo no est incluida en los clculos.
        datos_venta_producto := OBTENER_DATOS_VENTA_PRODUCTO (e_cod_ean, fecha_actual, hora_actual);
     
         -- Inserta la venta en la tabla de ventas.
        INSERT INTO VENTAS ("VENTA_ID", "TIENDA_ID", "COD_EAN", "FECHA", "HORA", "UD_VENDIDAS", "CTD_TOTAL", "PB_TOTAL",
        "BN_TOTAL","PB_UNITARIO","BN_UNITARIO")
        VALUES (seq_ventas.nextval, e_tienda_id, e_cod_ean, fecha_actual, hora_actual, e_ud_vendidas, (datos_venta_producto(1)+ e_ud_vendidas),
        (datos_venta_producto (2)+ e_precio_bruto * e_ud_vendidas), (datos_venta_producto(3)+ e_beneficio_neto * e_ud_vendidas),
        e_precio_bruto, e_beneficio_neto);
     
        -- Actualiza las estadsticas de direccin.
        PKG_EST_DIR.ACTLZR_EST_TOTAL(e_tienda_id, e_cod_ean, fecha_actual, hora_actual, e_ud_vendidas,e_precio_bruto,e_beneficio_neto, rsp_temp);
        PKG_EST_DIR.ACTLZR_EST_ANUAL(e_tienda_id, e_cod_ean, fecha_actual, hora_actual, e_ud_vendidas,e_precio_bruto,e_beneficio_neto, rsp_temp);
     

        -- Si la insercin se ha efectuado correctamente da el valor OK a rsp y actualiza el registro de log    
        rsp :='OK';
        PKG_CONTROL.ACTUALIZA_LOG_SALIDA (id_log,rsp);

        COMMIT;
    
      end if;

      EXCEPTION
       -- Si no se ha podido abrir registro para el log informa al usuario.      
      WHEN problema_log THEN
        rsp:= 'ERROR. No se puede guardar registro de transaccin.';
      -- Si no existe la tienda.
      WHEN no_existe_tienda THEN
        rsp:='ERROR. No existe la tienda.';
        PKG_CONTROL.ACTUALIZA_LOG_SALIDA (id_log,rsp);
      -- sin no existe el producto lanza en error y avisa de este hecho.
      WHEN no_existe_producto THEN
        rsp:='ERROR. No existe el producto.';
        PKG_CONTROL.ACTUALIZA_LOG_SALIDA (id_log,rsp);
      WHEN OTHERS THEN
        rsp:='ERROR ' || SUBSTR(SQLERRM,1,200);
        ROLLBACK;
        PKG_CONTROL.ACTUALIZA_LOG_SALIDA (id_log,rsp);

END INSERTAR_VENTA_PRUEBAS;

END PKG_VENTA;

/
spool off;
