/*
	NOMBRE DEL PAQUETE: PKG_CONSULTAS
	
  FUNCIONALIDAD: Proporciona las operaciones necesarias para realizar las consultas 
  con informacin estadstica sobre la base de datos.
  Se incluyen tanto las consultas que se realizarn sobre las tablas de VENTAS, PRODUCTS y TIENDAS
  como aquellas consultas que se realizarn sobre la tabla de ESTADISTICAS_DIRECCION

*/

spool 918_Implementacion_PKG_CONSULTAS.log;

------------------------------------------------
------------------- CABECERA -------------------
------------------------------------------------

CREATE OR REPLACE PACKAGE PKG_CONSULTAS IS
	NOMPAQUETE CHAR(20) := 'PKG_CONSULTAS';

-----------------------------
-------- CONSULTA 1 ---------
-----------------------------

/* 
   Funcin que devuelve el beneficio neto total de la cadena para un ao y mes concretos.
   @param e_ano: El ao para el que se quiere determinar el dato.
   @param e_mes: El mes para el que se quiere determinar el dato.
   @return El beneficio neto total para el periodo de referencia
           Si no hay ventas devuelve 0.
*/
FUNCTION TOTAL_VPERIODO_CADENA(
   e_ano IN NUMBER
,  e_mes IN NUMBER
) 
RETURN NUMBER;

/* 
   Procedimiento que devuelve un listado de las tiendas de la cadena con:
   El identificador de la tienda (TIENDA).
   El total de unidades que ha vendido dicha tienda de todos los productos (UNIDADES).
   El nmero de productos diferentes que ha vendido (PRODUCTOS).
   El beneficio neto total que ha obtenido (BENEFICIO).
   El porcentaje de beneficio que aporta la tienda en relacin con los beneficios totales de la cadena.
   El beneficio neto dividido por el nmero de empleados.
   Todo ello para el mes y ao de referencia y que recibe como parmetro.
   Ordenado por Beneficio neto obtenido de forma descendente.
   @param e_ano: El ao para el que se quiere determinar el dato.
   @param e_mes: El mes para el que se quiere determinar el dato.
   @return Un listado de tipo T_TABLA_VENTAS_TIENDA
   @return rsp: "OK" si la consulta se ha desarrollado sin problemas.
                "ERROR + TIPO ERROR" si no se podido ejecutar la consulta.
*/
PROCEDURE LISTADO_VENTAS_TIENDAS (
   e_ano IN NUMBER
,  e_mes IN NUMBER
,  listado OUT T_TABLA_VENTA_TIENDA
,  rsp OUT STRING
);


-----------------------------
-------- CONSULTA 2 ---------
-----------------------------


/* 
   Procedimiento que devuelve un listado de los productos de la cadena con:
   El cdigo EAN del producto (PRODUCTO).
   La descripcin del producto (DESCRIPCION).
   El nmero de unidades vendidas(VENTAS).
   El beneficio neto que ha generado el producto (BENEFICIO).
   La tienda que ms unidades ha vendido de ese producto (TIENDA).
   Las unidades de ese producto que ha vendido la tienda anterior (UNIDADES).
   Todo ello para el mes y ao de referencia y que recibe como parmetro.
   Ordenado por Beneficio neto obtenido de forma descendente.
   @param e_ano: El ao para el que se quiere determinar el dato.
   @param e_mes: El mes para el que se quiere determinar el dato.
   @return Un listado de tipo T_TABLA_VENTAS_PRODUCTO.
   @return rsp: "OK" si la consulta se ha desarrollado sin problemas. 
                "ERROR + TIPO ERROR" si no se podido ejecutar la consulta.
*/
PROCEDURE LISTADO_VENTAS_PRODUCTOS (
   e_ano IN NUMBER
,  e_mes IN NUMBER
,  listado OUT T_TABLA_VENTA_PRODUCTO
,  rsp OUT STRING
);


-----------------------------
-------- CONSULTA 3 ---------
-----------------------------


/* 
   Procedimiento que devuelve un listado de los das del mes con:
   Beneficio total neto obtenido por la cadena cada da (D_BENEFICIO).
   Identificador del producto ms vendido cada da (PRODUCTO).
   Unidades vendidas de ese producto ese da(UNIDADES).
   Tienda que ms beneficio neto ha generado (TIENDA).
   El beneficio neto que ha generado la tienda ese da (T_BENEFICIO).
   Todo ello para el mes y ao de referencia y que recibe como parmetro.
   @param e_ano: El ao para el que se quiere determinar el dato.
   @param e_mes: El mes para el que se quiere determinar el dato.
   @return Un listado de tipo T_TABLA_VENTA_DIA.
   @return rsp: "OK" si la consulta se ha desarrollado sin problemas. 
                "ERROR + TIPO ERROR" si no se podido ejecutar la consulta.
*/
PROCEDURE LISTADO_VENTAS_DIARIAS (
   e_ano IN NUMBER
,  e_mes IN NUMBER
,  listado OUT T_TABLA_VENTA_DIA
,  rsp OUT STRING
);


-----------------------------
---- CONSULTAS DIRECCION ----
-----------------------------

/*
Procedimiento que, haciendo un SELECT sobre la tabla ESTADISTICAS_DIRECCION, devuelve los valores de la fila
correspondiente al periodo indicado en una variable de tipo ESTADISTICAS_DIRECCION%ROWTYPE que puede ser leda por otro procedimiento.
@param e_periodo: El periodo objeto del informe.
@return los valores de la fila objeto de consulta.
@return rsp: "OK" si la consulta se ha desarrollado sin problemas. 
             "ERROR + TIPO ERROR" si no se podido ejecutar la consulta.
*/
PROCEDURE CONSULTA_ESTADISTICA (
   e_periodo IN NUMBER
,  listado OUT ESTADISTICAS_DIRECCION%ROWTYPE
,  rsp OUT STRING
);

/*
Procedimiento que, haciendo un SELECT sobre la tabla ESTADISTICAS_DIRECCION, devuelve los valores de la lnea 
con el identificador 0 en una variable de tipo ESTADISTICAS_DIRECCION%ROWTYPE que puede ser leda por otro procedimiento.
@param e_periodo: El periodo objeto del informe.
@return los valores de la fila objeto de consulta.
@return rsp: "OK" si la consulta se ha desarrollado sin problemas. 
             "ERROR + TIPO ERROR" si no se podido ejecutar la consulta.
*/
PROCEDURE CONSULTA_ESTADISTICA_GLOBAL (
   listado OUT ESTADISTICAS_DIRECCION%ROWTYPE
,  rsp OUT STRING
);


/*
Procedimiento que, haciendo un SELECT sobre la tabla ESTADISTICAS_DIRECCION, devuelve los valores de la lnea 
con el identificador correspondiente al ao de la consulta con el formato YYYY en una variable de tipo ESTADISTICAS_DIRECCION%ROWTYPE 
que puede ser leda por otro procedimiento.
@param e_periodo: El periodo objeto del informe.
@return los valores de la fila objeto de consulta.
@return rsp: "OK" si la consulta se ha desarrollado sin problemas. 
             "ERROR + TIPO ERROR" si no se podido ejecutar la consulta.
*/
PROCEDURE CONSULTA_ESTADISTICA_ANUAL (
   e_periodo IN NUMBER
,  listado OUT ESTADISTICAS_DIRECCION%ROWTYPE
,  rsp OUT STRING
);

end PKG_CONSULTAS;
/

----------------------------------------------
------------------- CUERPO -------------------
----------------------------------------------
CREATE OR REPLACE PACKAGE BODY PKG_CONSULTAS IS 

-----------------------------
-------- CONSULTA 1 ---------
-----------------------------

FUNCTION TOTAL_VPERIODO_CADENA(
   e_ano IN NUMBER
,  e_mes IN NUMBER
) 
RETURN NUMBER 

AS 

rdo NUMBER(10,2);

BEGIN

SELECT SUM(UD_VENDIDAS * BN_UNITARIO) AS BENEFICIO INTO rdo
FROM VENTAS
WHERE TO_CHAR(FECHA,'YYYY') = e_ano
AND TO_CHAR (FECHA,'MM') = e_mes;

-- Si no hay valores devuelve 0.
IF sql%ROWCOUNT = 0 THEN
  rdo := 0;
END IF;
      

RETURN rdo;
  
END TOTAL_VPERIODO_CADENA;


PROCEDURE LISTADO_VENTAS_TIENDAS (
   e_ano IN NUMBER
,  e_mes IN NUMBER
,  listado OUT T_TABLA_VENTA_TIENDA
,  rsp OUT STRING
) AS 

 /* 
    Se declara un cursor para que devuelva un listado de tiendas que incluya para cada una:
    TIENDA_ID (TIENDA)
    el total de unidades que ha vendido dicha tienda de todos los productos (UNIDADES)
    el nmero de productos diferentes que ha vendido (PRODUCTOS)
    el beneficio neto total que ha obtenido (BENEFICIO)
    el nmero de empleados que tiene (NUMERO_EMPLEADOS)
    Todo ello para el mes y ao de referencia y que recibe como parmetro.
    Ordenado por Beneficio neto obtenido.
 */
 CURSOR CT IS SELECT L.TIENDA, L.UNIDADES, L.PRODUCTOS, L.BENEFICIO, E.NUMERO_EMPLEADOS
              FROM(
                  -- listado de ventas para un ao y mes concreto agrupado por tiendas con los datos de venta necesarios
                  (SELECT TIENDA_ID AS TIENDA, SUM(UD_VENDIDAS) AS UNIDADES
                   , COUNT (DISTINCT COD_EAN) AS PRODUCTOS, SUM(UD_VENDIDAS * BN_UNITARIO) AS BENEFICIO
                   FROM VENTAS
                   WHERE TO_CHAR(FECHA,'YYYY') = e_ano
                   AND TO_CHAR (FECHA,'MM')= e_mes
                   GROUP BY TIENDA_ID) L
         
                   LEFT JOIN  -- Left join por si no hubiera empleados en dicha tienda. 
 
                   -- Listado de las tiendas con el nmero de empleados que tienen para ese mes y ao.
                  (SELECT TIENDA_ID, NUMERO_EMPLEADOS FROM N_EMPLEADOS
                   WHERE ANO = e_ano
                   AND MES = e_mes) E
               
                    ON L.TIENDA = E.TIENDA_ID
                  )
                  ORDER BY BENEFICIO DESC;
 
   -- Variable para almacenar el valor del LOG_ID que le corresponda a la llamada al procedimiento.
   id_log INTEGER;
   -- Variable para registrar el beneficio resultado total de la cadena de tiendas para ese mes.
   resultado_total NUMBER;
   -- Variable para recoger los datos que devuelve el cursor.
   rec T_VENTA_TIENDA := T_VENTA_TIENDA(0,0,0,0,0,0);
   -- Variable para almacenar el resultado de la consulta en una tabla de tipo T_VENTAS_TIENDA.
   resultado1 T_TABLA_VENTA_TIENDA := T_TABLA_VENTA_TIENDA();
   -- Controlador de registro de acceso a la tabla de resultado1.
   c NUMBER := 0;
   
   -- Excepcin en el caso de que falle el registro de LOG.
   problema_log EXCEPTION;

 BEGIN
  
   -- Registra en la tabla de LOGS la llamada al procedimiento.   
   id_log := PKG_CONTROL.INSERTA_LOG('LISTADO_VENTAS_TIENDAS', 'Ao: '||e_ano||', Mes: '||e_mes); 

   -- Si hay algn problema al insertar el LOG lanza una excepcin definida por el usuario.
   -- Si no hay problema al insertar el LOG prosigue con la emisin del informe.
   IF id_log = 0 THEN
      RAISE problema_log;
   
   ELSE
     
      -- Se calcula el resultado de la cadena y se almacena para su uso posterior en los clculos.
      resultado_total := TOTAL_VPERIODO_CADENA(e_ano, e_mes);

      -- Se abre el cursor
      OPEN CT;
    
       -- Se recorre el cursor para modificar los campos calculados   
       -- Termina cuando no hay siguiente registro
       -- Para cada registro aade una lnea en resultado1 y graba los datos almacenados en el cursor,
       -- calcula el % de las ventas de la tienda sobre el beneficio total de la cadena y lo aade,
       -- calcula el beneficio neto entre el nmero de empleados
      LOOP
        FETCH CT INTO rec.tienda, rec.unidades, rec.productos, rec.beneficio, rec.empleados;
        EXIT WHEN CT%NOTFOUND;
        resultado1.EXTEND;
        c := c+1;
        resultado1 (c) :=rec;
        resultado1 (c).porcentaje := (resultado1(c).beneficio/resultado_total*100);
        resultado1 (c).empleados := (resultado1(c).beneficio/resultado1(c).empleados);
      END LOOP;
      
      -- Cierra cursor
      CLOSE CT;
    
      -- Asigna el resultado a la variable de salida y el msg de rsp
      listado := resultado1;
   
      -- Si la insercin se ha efectuado correctamente da el valor OK a rsp y actualiza el registro de log    
      rsp :='OK';
      PKG_CONTROL.ACTUALIZA_LOG_SALIDA (id_log,rsp);
      
    END IF;
    
    EXCEPTION
       -- Si no se ha podido abrir registro para el log informa al usuario.
       WHEN problema_log THEN
         rsp:= 'ERROR. No se puede guardar el log de la transaccin.';
       -- Si el listado no tiene valores.
       WHEN NO_DATA_FOUND THEN
         rsp:= 'ERROR. No ha seleccionado ningn valor.';
         PKG_CONTROL.ACTUALIZA_LOG_SALIDA (id_log,rsp);
       --Para cualquier otro error informa en la tabla de LOGS del error.
       WHEN OTHERS THEN
         rsp:='ERROR ' || SUBSTR(SQLERRM,1,200);
         PKG_CONTROL.ACTUALIZA_LOG_SALIDA (id_log,rsp);
 
END LISTADO_VENTAS_TIENDAS;

-----------------------------
---- consulta 2--------
-----------------------------

PROCEDURE LISTADO_VENTAS_PRODUCTOS (
   e_ano IN NUMBER
,  e_mes IN NUMBER
,  listado OUT T_TABLA_VENTA_PRODUCTO
,  rsp OUT STRING
)
AS 

 /* 
    Se declara un cursor para que devuelva un listado de productos que incluya para cada uno:
    El cdigo EAN del producto (COD_EAN)
    La descripcin del producto (DESCRIPCION)
    El nmero de unidades vendidas(VENTAS)
    El beneficio neto que ha generado el producto (BENEFICIO)
    La tienda que ms unidades ha vendido de ese producto (TIENDA)
    Las unidades de ese producto que ha vendido la tienda anterior (UNIDADES)
    Todo ello para el mes y ao de referencia y que recibe como parmetro.
    Ordenado por Beneficio neto obtenido.
 */ 
               -- Para hacer la consulta se toma una seleccin de todas las ventas para ese ao y mes
               -- se muestran agrupadas para cada tienda y cdigo ean de producto
  CURSOR CT IS WITH TBL1 AS (SELECT TIENDA_ID, COD_EAN, SUM (UD_VENDIDAS) as UNIDADES FROM VENTAS
                             WHERE TO_CHAR (FECHA,'YYYY') = e_ano
                             AND TO_CHAR (FECHA, 'MM') = e_mes
                             GROUP BY TIENDA_ID, COD_EAN)
 
               SELECT TBL2.COD_EAN, TBL2.DESCRIPCION, TBL2.VENTAS, TBL2.BENEFICIO, TBL3.TIENDA_ID, TBL3.UNIDADES 
               FROM(
                   -- Ofrece un listado de productos con el cdigo, la descripcin as como las unidades vendidas y beneficio de cada producto.
                   -- sobre la tabla de ventas para el periodo indicado
                   (SELECT V.COD_EAN, P.DESCRIPCION, SUM(V.UD_VENDIDAS) AS VENTAS,
                    SUM (V.UD_VENDIDAS * V.BN_UNITARIO) AS BENEFICIO
                    FROM VENTAS V
                    JOIN PRODUCTOS P
                    ON V.COD_EAN = P.COD_EAN
                    WHERE TO_CHAR (V.FECHA,'YYYY') = e_ano
                    AND TO_CHAR (V.FECHA, 'MM') = e_mes
                    GROUP BY V.COD_EAN, P.DESCRIPCION) TBL2
        
                    JOIN
  
                   -- Ofrece un listado con el mximo de ventas para cada tienda y cdigo de producto.
                   (SELECT * FROM TBL1
                    WHERE (COD_EAN, UNIDADES) IN (SELECT COD_EAN, MAX (UNIDADES)
                                                  FROM TBL1 GROUP BY COD_EAN)
                    ORDER BY COD_EAN) TBL3
                      
                    ON TBL2.COD_EAN = TBL3.COD_EAN         
                    )
               ORDER BY BENEFICIO DESC, COD_EAN;

     -- Variable para almacenar el valor del LOG_ID que le corresponda a la llamada al procedimiento
   id_log INTEGER;
   -- Variable para recoger los datos que devuelve el cursor.
   rec T_VENTA_PRODUCTO := T_VENTA_PRODUCTO (0,'', 0, 0, 0, 0); 
   -- Variable para almacenar el resultado de la consulta en una tabla de tipo T_VENTAS_PRODUCTO.
   resultado1 T_TABLA_VENTA_PRODUCTO := T_TABLA_VENTA_PRODUCTO();
   -- controlador de registro de acceso a la tabla de resultado1.
   c NUMBER := 0;
  
   -- Excepcin en el caso de que falle el registro de LOG.
   problema_log EXCEPTION;

BEGIN

   -- Registra en la tabla de LOGS la llamada al procedimiento.   
   id_log := PKG_CONTROL.INSERTA_LOG('LISTADO_VENTAS_PRODUCTOS', 'Ao: '||e_ano||', Mes: '||e_mes); 

   -- Si hay algn problema al insertar el LOG lanza una excepcin definida por el usuario.
   -- Si no hay problema al insertar el LOG prosigue con la emisin del informe.
   IF id_log = 0 THEN
      RAISE problema_log;
    
   ELSE
    
      -- Se abre el cursor
      OPEN CT;
           
      -- Se recorre el cursor y se almacena los valores de cada registro en la variable de salida.
      LOOP
        FETCH CT INTO rec.producto, rec.descripcion, rec.ventas, rec.beneficio, rec.tienda, rec.unidades;
        EXIT WHEN CT%NOTFOUND;
        resultado1.EXTEND;
        c := c+1;
        resultado1(c) :=rec;
      END LOOP;
  
      -- Cierra cursor
      CLOSE CT;
    
      -- Asigna el resultado a la variable de salida
      listado := resultado1;
   
      -- Si la insercin se ha efectuado correctamente da el valor OK a rsp y actualiza el registro de log    
      rsp :='OK';
      PKG_CONTROL.ACTUALIZA_LOG_SALIDA (id_log,rsp);

   END IF;
   
   EXCEPTION
     -- Si no se ha podido abrir registro para el log informa al usuario 
     WHEN problema_log THEN
       rsp:= 'ERROR. No se puede guardar el log de la transaccin.';
     -- Si el listado no tiene valores
     WHEN NO_DATA_FOUND THEN
        rsp:= 'ERROR. No ha seleccionado ningn valor.';
        PKG_CONTROL.ACTUALIZA_LOG_SALIDA (id_log,rsp);
     --Para cualquier otro error informa en la tabla de LOGS del error
     WHEN OTHERS THEN
       rsp:='ERROR ' || SUBSTR(SQLERRM,1,200);
       PKG_CONTROL.ACTUALIZA_LOG_SALIDA (id_log,rsp);

END LISTADO_VENTAS_PRODUCTOS;

-----------------------------
-------- CONSULTA 3 ---------
-----------------------------

PROCEDURE LISTADO_VENTAS_DIARIAS (
   e_ano IN NUMBER
,  e_mes IN NUMBER
,  listado OUT T_TABLA_VENTA_DIA
,  rsp OUT STRING
)
AS 

 /* 
    Se declara un cursor para que devuelva un listado de los das que incluya para cada uno:
    El da (FECHA)
    Beneficio total neto obtenido por la cadena cada da (D_BENEFICIO)
    Identificador del producto ms vendido cada da (PRODUCTO)
    Unidades vendidas de ese producto ese da(UNIDADES)
    Tienda que ms beneficio neto ha generado (TIENDA)
    El beneficio neto que ha generado la tienda ese da (T_BENEFICIO)
    Todo ello para el mes y ao de referencia y que recibe como parmetro.
    Ordenado por fecha.
 */ 

  CURSOR CT IS WITh 
  
                 -- Todas las ventas para ese ao y mes agrupadas por tienda y cdigo de producto.
                 VENTAS_DIA_PRODUCTO AS (SELECT FECHA, COD_EAN, SUM(UD_VENDIDAS) as UNIDADES FROM VENTAS
                                         where to_char (FECHA,'YYYY') = e_ano
                                         and to_char(FECHA, 'MM')= e_mes
                                         GROUP BY FECHA, COD_EAN),

                 -- Todas las ventas para ese ao y mes agrupadas por fecha y tienda.
                 VENTAS_DIA_TIENDA AS (SELECT FECHA, TIENDA_ID, SUM (UD_VENDIDAS * BN_UNITARIO) AS BENEFICIO_TIENDA FROM VENTAS
                                       where to_char (FECHA,'YYYY') = e_ano
                                       and to_char(FECHA, 'MM') = e_mes
                                       GROUP BY FECHA, TIENDA_ID)

            SELECT  BN_DIARIO.FECHA, BN_DIARIO.BENEFICIO, P_MAS_VENDIDO_DIARIO.COD_EAN, P_MAS_VENDIDO_DIARIO.UNIDADES,
                    T_MAS_VENTAS_DIARIO.TIENDA_ID, T_MAS_VENTAS_DIARIO.BENEFICIO_TIENDA
                    FROM
                    -- Listado de das con su beneficio
                   (SELECT FECHA, SUM(UD_VENDIDAS * BN_UNITARIO) AS BENEFICIO FROM VENTAS
                    WHERE to_char (FECHA,'YYYY') = e_ano
                    AND to_char(FECHA, 'MM') = e_mes
                    GROUP BY FECHA) BN_DIARIO
                    
                    JOIN
                     -- el producto ms vendido en cada da
                   (SELECT * FROM VENTAS_DIA_PRODUCTO
                    WHERE (FECHA, UNIDADES) IN (SELECT FECHA, MAX(UNIDADES) 
                                                FROM VENTAS_DIA_PRODUCTO
                                                GROUP BY FECHA)
                    ) P_MAS_VENDIDO_DIARIO
              
                    ON BN_DIARIO.FECHA = P_MAS_VENDIDO_DIARIO.FECHA
              
                    JOIN 
                    -- Tienda que ms beneficio ha generado cada da.            
                   (SELECT * FROM VENTAS_DIA_TIENDA
                    WHERE (FECHA, BENEFICIO_TIENDA) IN (SELECT FECHA, MAX (BENEFICIO_TIENDA) 
                                                        FROM VENTAS_DIA_TIENDA 
                                                        GROUP BY FECHA)
                   ) T_MAS_VENTAS_DIARIO

              
                    ON BN_DIARIO.FECHA = T_MAS_VENTAS_DIARIO.FECHA
              
              ORDER BY BN_DIARIO.FECHA;



   -- Variable para almacenar el valor del LOG_ID que le corresponda a la llamada al procedimiento.
   id_log INTEGER;
   -- Variable para recoger los datos que devuelve el cursor.
   rec T_VENTA_DIA := T_VENTA_DIA ('', 0, 0, 0, 0, 0); 
   -- Variable para almacenar el resultado de la consulta en una tabla de tipo T_VENTAS_PRODUCTO.
   resultado1 T_TABLA_VENTA_DIA := T_TABLA_VENTA_DIA();
   -- controlador de registro de acceso a la tabla de resultado1.
   c NUMBER := 0;
  
   -- Excepcin en el caso de que falle el registro de LOG.
   problema_log EXCEPTION;

BEGIN

   -- Registra en la tabla de LOGS la llamada al procedimiento   
   id_log := PKG_CONTROL.INSERTA_LOG('LISTADO_VENTAS_DIARIAS', 'Ao: '||e_ano||', Mes: '||e_mes); 

   -- Si hay algn problema al insertar el LOG lanza una excepcin definida por el usuario
   -- Si no hay problema al insertar el LOG prosigue con la emisin del informe
   IF id_log = 0 THEN
      RAISE problema_log;
    
   ELSE
    
      -- Se abre el cursor
      OPEN CT;
           
      -- Se recorre el cursor y se almacena los valores de cada registro en la variable de salida.
      LOOP
        FETCH CT INTO rec.dia, rec.d_beneficio, rec.producto, rec.unidades, rec.tienda, rec.t_beneficio;
        EXIT WHEN CT%NOTFOUND;
        resultado1.EXTEND;
        c := c+1;
        resultado1(c) :=rec;
      END LOOP;
  
      -- Cierra cursor
      CLOSE CT;
    
      -- Asigna el resultado a la variable de salida.
      listado := resultado1;
   
      -- Si la insercin se ha efectuado correctamente da el valor OK a rsp y actualiza el registro de log.   
      rsp :='OK';
      PKG_CONTROL.ACTUALIZA_LOG_SALIDA (id_log,rsp);

   END IF;
   
   EXCEPTION
     -- Si no se ha podido abrir registro para el log informa al usuario. 
     WHEN problema_log THEN
       rsp:= 'ERROR. No se puede guardar el log de la transaccin.';
     -- Si el listado no tiene valores.
     WHEN NO_DATA_FOUND THEN
        rsp:= 'ERROR. No ha seleccionado ningn valor.';
        PKG_CONTROL.ACTUALIZA_LOG_SALIDA (id_log,rsp);
     --Para cualquier otro error informa en la tabla de LOGS del error.
     WHEN OTHERS THEN
       rsp:='ERROR ' || SUBSTR(SQLERRM,1,200);
       PKG_CONTROL.ACTUALIZA_LOG_SALIDA (id_log,rsp);

END LISTADO_VENTAS_DIARIAS;

-----------------------------
---- CONSULTAS DIRECCION ----
-----------------------------

PROCEDURE CONSULTA_ESTADISTICA (
   e_periodo IN NUMBER
,  listado OUT ESTADISTICAS_DIRECCION%ROWTYPE
,  rsp OUT STRING
)

AS
   -- Variable para almacenar el valor del LOG_ID que le corresponda a la llamada al procedimiento.
   id_log INTEGER;

   -- Excepcin en el caso de que falle el registro de LOG.
   problema_log EXCEPTION;   
   
BEGIN

   -- Registra en la tabla de LOGS la llamada al procedimiento.   
   id_log := PKG_CONTROL.INSERTA_LOG('CONSULTA_ESTADISTICA', 'Ao: '||e_periodo); 

   -- Si hay algn problema al insertar el LOG lanza una excepcin definida por el usuario.
   -- Si no hay problema al insertar el LOG prosigue con la emisin del informe.
   IF id_log = 0 THEN
      RAISE problema_log;
    
   ELSE
   
    SELECT * INTO listado FROM ESTADISTICAS_DIRECCION
    WHERE EST_ID = e_periodo;
    
  END IF;
  
    -- Si el procedimiento se ha ejecutado correctamente da el valor OK a rsp y actualiza el registro de log.    
    rsp :='OK';
    PKG_CONTROL.ACTUALIZA_LOG_SALIDA (id_log,rsp);
    
    EXCEPTION
     -- Si no se ha podido abrir registro para el log informa al usuario 
     WHEN problema_log THEN
       rsp:= 'ERROR No se puede guardar el log de la transaccin';
     WHEN NO_DATA_FOUND THEN
        listado.EST_ID := 0;
        listado.BN_AGREGADO := 0;
        listado.ID_TIENDA_MAX := 0;
        listado.BN_TIENDA_MAX := 0;
        listado.COD_EAN_MAX := 0;
        listado.CTD_UNIDADES_EAN_MAX := 0;
        listado.H_MAX := 0;
        listado.CTD_UNIDADES_H_MAX := 0;
        listado.H_MIN := 0;
        listado.CTD_UNIDADES_H_MIN := 0;
        listado.D_MAX := 0;
        listado.CTD_UNIDADES_D_MAX := 0;
        listado.D_MIN := 0;
        listado.CTD_UNIDADES_D_MIN := 0;
        listado.CIUDAD_MAX := '0';
        listado.BN_CIUDAD_MAX := 0;
        listado.PORCNTJ_BENFCS_VIRTLS := 0;
        
        -- Si el procedimiento se ha ejecutado correctamente da el valor OK  rsp y actualiza el registro de log.    
        rsp :='OK';
        PKG_CONTROL.ACTUALIZA_LOG_SALIDA (id_log,rsp);
     
     --Para cualquier otro error informa en la tabla de LOGS del error.
     WHEN OTHERS THEN
       rsp:='ERROR ' || SUBSTR(SQLERRM,1,200);
       PKG_CONTROL.ACTUALIZA_LOG_SALIDA (id_log,rsp);   
    

END CONSULTA_ESTADISTICA;


PROCEDURE CONSULTA_ESTADISTICA_GLOBAL (
   listado OUT ESTADISTICAS_DIRECCION%ROWTYPE
,  rsp OUT STRING
)

AS

   -- Variable para almacenar el valor del LOG_ID que le corresponda a la llamada al procedimiento.
   id_log INTEGER;

   -- Excepcin en el caso de que falle el registro de LOG.
   problema_log EXCEPTION;   
   
BEGIN

   -- Registra en la tabla de LOGS la llamada al procedimiento   
   id_log := PKG_CONTROL.INSERTA_LOG('CONSULTA_ESTADISTICA_GLOBAL','No hay parmetros.'); 
 
   -- Si hay algn problema al insertar el LOG lanza una excepcin definida por el usuario.
   -- Si no hay problema al insertar el LOG prosigue con la emisin del informe.
   IF id_log = 0 THEN
      RAISE problema_log;
    
   ELSE

      CONSULTA_ESTADISTICA(0, listado, rsp);
      
   END IF;
  
   -- Si la insercin se ha efectuado correctamente da el valor OK  rsp y actualiza el registro de log  .  
   rsp :='OK';
   PKG_CONTROL.ACTUALIZA_LOG_SALIDA (id_log,rsp);

   EXCEPTION
    -- Si no se ha podido abrir registro para el log informa al usuario. 
    WHEN problema_log THEN
      rsp:= 'ERROR. No se puede guardar el log de la transaccin.';
      --Para cualquier otro error informa en la tabla de LOGS del error.
    WHEN OTHERS THEN
      rsp:='ERROR ' || SUBSTR(SQLERRM,1,200);
      PKG_CONTROL.ACTUALIZA_LOG_SALIDA (id_log,rsp);  
       
END CONSULTA_ESTADISTICA_GLOBAL;




PROCEDURE CONSULTA_ESTADISTICA_ANUAL (
   e_periodo IN NUMBER
,  listado OUT ESTADISTICAS_DIRECCION%ROWTYPE
,  rsp OUT STRING
)

AS

   -- Variable para almacenar el valor del LOG_ID que le corresponda a la llamada al procedimiento.
   id_log INTEGER;

   -- Excepcin en el caso de que falle el registro de LOG.
   problema_log EXCEPTION;   
   
BEGIN

   -- Registra en la tabla de LOGS la llamada al procedimiento   
   id_log := PKG_CONTROL.INSERTA_LOG('CONSULTA_ESTADISTICA_ANUAL','Periodo: '||e_periodo); 
 
   -- Si hay algn problema al insertar el LOG lanza una excepcin definida por el usuario.
   -- Si no hay problema al insertar el LOG prosigue con la emisin del informe.
   IF id_log = 0 THEN
      RAISE problema_log;
    
   ELSE

      CONSULTA_ESTADISTICA(e_periodo, listado, rsp);
      
   END IF;
  
   -- Si la insercin se ha efectuado correctamente da el valor OK  rsp y actualiza el registro de log.    
   rsp :='OK';
   PKG_CONTROL.ACTUALIZA_LOG_SALIDA (id_log,rsp);

   EXCEPTION
    -- Si no se ha podido abrir registro para el log informa al usuario. 
    WHEN problema_log THEN
      rsp:= 'ERROR. No se puede guardar el log de la transaccin.';
      --Para cualquier otro error informa en la tabla de LOGS del error.
    WHEN OTHERS THEN
      rsp:='ERROR ' || SUBSTR(SQLERRM,1,200);
      PKG_CONTROL.ACTUALIZA_LOG_SALIDA (id_log,rsp);  
       
END CONSULTA_ESTADISTICA_ANUAL;

END PKG_CONSULTAS;

/
spool off;

